@@ -673,7 +673,8 @@ magnifier_get_display_rect_bounds (Magnifier *magnifier, GNOME_Magnifier_RectBou
     }
 }
 
-static void magnifier_adjust_source_size (Magnifier *magnifier)
+static void
+magnifier_adjust_source_size (Magnifier *magnifier)
 {
 	GNOME_Magnifier_RectBounds rect_bounds;	
 	gdouble vfract_top, vfract_bottom, hfract_left, hfract_right;
@@ -1885,6 +1886,11 @@ magnifier_init (Magnifier *magnifier)
 			       gdk_display_get_screen (magnifier->target_display, 
 						       magnifier->target_screen_num));
 	magnifier_init_cursor_set (magnifier, "default");
+
+	mag_timing.process = g_timer_new ();
+	mag_timing.frame = g_timer_new ();
+	mag_timing.scale = g_timer_new ();
+	mag_timing.idle = g_timer_new ();
 #ifdef DEBUG_CLIENT_CALLS
 	client_debug = (g_getenv ("MAG_CLIENT_DEBUG") != NULL);
 #endif
