@@ -59,6 +59,7 @@ static GObjectClass *parent_class = NULL;
 
 enum {
 	ZOOM_REGION_MANAGED_PROP,
+	ZOOM_REGION_POLL_MOUSE_PROP,
 	ZOOM_REGION_SMOOTHSCROLL_PROP,
 	ZOOM_REGION_INVERT_PROP,
 	ZOOM_REGION_SMOOTHING_PROP,
@@ -81,6 +82,7 @@ enum {
 gchar* prop_names[ZOOM_REGION_EXIT_MAGNIFIER + 1] = 
 {
     "MANAGED",
+    "POLLMOUSE"
     "SMOOTHSCROLL",
     "INVERT",
     "SMOOTHING",
@@ -183,6 +185,29 @@ _debug_announce_rect (char *msg, GdkRectangle rect)
 		 msg, rect.x, rect.y, rect.x + rect.width, rect.y + rect.height);
 }
 
+#ifdef TEST_XTST_CURSOR
+static Cursor *
+_get_x_cursors (Display *display)
+{
+	int n_x_font_cursors = XC_num_glyphs; 
+	int dummy;
+	if (!XTestQueryExtension (display, &dummy, &dummy,&dummy, &dummy))
+		fprintf (stderr, "WARNING: XTEST extension not available!\n");
+	if (x_cursors == NULL && display) {
+		int i;
+		x_cursors = g_new0 (Cursor, XC_num_glyphs);
+		for (i = 2; i < n_x_font_cursors; i += 2)
+		{
+			if (i == 30)
+				x_cursors[i] = XCreateFontCursor (display, XC_xterm);
+			else
+				x_cursors[i] = XCreateFontCursor (display, i);
+		}
+	}
+	return x_cursors;
+}
+#endif
+
 static gboolean
 _diff_pixbufs (const GdkPixbuf *a, const GdkPixbuf *b)
 {
@@ -542,22 +567,34 @@ zoom_region_clip_to_exposed_target (ZoomRegion *zoom_region,
 	GNOME_Magnifier_RectBounds onscreen_target, *source_area;
 	source_area = &zoom_region->priv->source_area;
 
-	onscreen_target.x1 = MAX (floor (zoom_region->priv->exposed_bounds.x1
-					 / zoom_region->xscale),
-					 source_area->x1);
-	onscreen_target.y1 = MAX (floor (zoom_region->priv->exposed_bounds.y1
-					 / zoom_region->yscale),
-					 source_area->y1);
-	onscreen_target.x2 = MIN (ceil (zoom_region->priv->exposed_bounds.x2
-					/ zoom_region->xscale),
-					source_area->x2);
-	onscreen_target.y2 = MIN (ceil (zoom_region->priv->exposed_bounds.y2
-					/ zoom_region->yscale),
-					source_area->y2);
+	onscreen_target.x1 = MAX (zoom_region->priv->exposed_bounds.x1 / zoom_region->xscale,
+				  source_area->x1);
+	onscreen_target.y1 = MAX (zoom_region->priv->exposed_bounds.y1 / zoom_region->yscale,
+				  source_area->y1);
+	onscreen_target.x2 = MIN (zoom_region->priv->exposed_bounds.x2 / zoom_region->xscale,
+				  source_area->x2);
+	onscreen_target.y2 = MIN (zoom_region->priv->exposed_bounds.y2 / zoom_region->yscale,
+				  source_area->y2);
 
 	return _rectangle_clip_to_bounds (area, &onscreen_target);
 }
 
+#ifdef NEEDED
+static GdkRectangle
+zoom_region_clip_to_exposed_bounds (ZoomRegion *zoom_region,
+				    GdkRectangle area)
+{
+	return _rectangle_clip_to_bounds (area, &zoom_region->priv->exposed_bounds);
+}
+
+static GdkRectangle
+zoom_region_clip_to_viewport (ZoomRegion *zoom_region,
+			      GdkRectangle area)
+{
+	return _rectangle_clip_to_bounds (area, &zoom_region->priv->exposed_viewport);
+}
+#endif
+
 static GdkRectangle
 zoom_region_clip_to_scaled_pixmap (ZoomRegion *zoom_region,
 				   GdkRectangle area)
@@ -603,8 +640,8 @@ zoom_region_source_rect_from_view_bounds (ZoomRegion *zoom_region,
 			       / zoom_region->xscale);
 	source_rect.y = floor ((view_bounds->y1 + zoom_region->priv->exposed_bounds.y1)
 				/ zoom_region->yscale);
-	source_rect.width = ceil ((view_bounds->x2 - view_bounds->x1) / zoom_region->xscale) + 1;
-	source_rect.height = ceil ((view_bounds->y2 - view_bounds->y1) / zoom_region->yscale) + 1;
+	source_rect.width = ceil ((view_bounds->x2 - view_bounds->x1) / zoom_region->xscale);
+	source_rect.height = ceil ((view_bounds->y2 - view_bounds->y1) / zoom_region->yscale);
 	return source_rect;
 }
 
@@ -631,8 +668,8 @@ zoom_region_source_rect_from_view_rect (ZoomRegion *zoom_region,
 			       / zoom_region->xscale);
 	source_rect.y = floor ((view_rect.y + zoom_region->priv->exposed_bounds.y1)
 				/ zoom_region->yscale);
-	source_rect.width = ceil (view_rect.width / zoom_region->xscale) + 1;
-	source_rect.height = ceil (view_rect.height / zoom_region->yscale) + 1;
+	source_rect.width = ceil (view_rect.width / zoom_region->xscale);
+	source_rect.height = ceil (view_rect.height / zoom_region->yscale);
 	return source_rect;
 }
 
@@ -1390,6 +1427,20 @@ zoom_region_set_cursor_pos (ZoomRegion *zoom_region, int x, int y)
 	}
 }
 
+#ifdef TEST_XTST_CURSOR
+static GdkWindow *
+zoom_region_source_root (ZoomRegion *zoom_region)
+{
+	GdkScreen *source_screen = NULL;
+	if (zoom_region->priv->source_drawable)
+		source_screen =
+			gdk_drawable_get_screen (zoom_region->priv->source_drawable);
+	if (source_screen)
+		return gdk_screen_get_root_window (source_screen);
+	else return NULL;
+}
+#endif
+
 static gboolean
 zoom_region_update_pointer (ZoomRegion *zoom_region, gboolean draw_cursor)
 {
@@ -1400,7 +1451,8 @@ zoom_region_update_pointer (ZoomRegion *zoom_region, gboolean draw_cursor)
 #ifdef ZOOM_REGION_DEBUG
 	g_assert (zoom_region->alive);
 #endif
-	if (!zoom_region->priv || !zoom_region->priv->parent)
+	if (!zoom_region->priv || !zoom_region->priv->parent 
+	    || !zoom_region->poll_mouse)
 	      return FALSE; 
 
 	magnifier = zoom_region->priv->parent;
@@ -1977,6 +2029,57 @@ zoom_region_time_frame(ZoomRegion *zoom_region, Magnifier *magnifier)
 	}
 }
 
+void
+exit_report(ZoomRegion *zoom_region)
+{
+	float frame_avg;
+	float x_scroll_incr, y_scroll_incr;
+	int width, height, x, y;
+
+	if (timing_test) {
+		width = (zoom_region->priv->exposed_viewport.x2 -
+			zoom_region->priv->exposed_viewport.x1) / zoom_region->xscale;
+		height = (zoom_region->priv->exposed_viewport.y2 -
+			zoom_region->priv->exposed_viewport.y1) / zoom_region->yscale;
+
+		frame_avg = mag_timing.frame_total / mag_timing.num_frame_samples;
+
+		x_scroll_incr = (float)mag_timing.dx_total / (float)mag_timing.num_line_samples;
+		y_scroll_incr = (float)mag_timing.dy_total / (float)mag_timing.num_line_samples;
+
+		gdk_drawable_get_size (GDK_DRAWABLE (zoom_region->priv->w->window),
+			&x, &y);
+
+		fprintf(stderr, "\nFinal Report:\n\n");
+
+		fprintf(stderr, "  Frames Processed         = %ld\n", 
+			mag_timing.num_frame_samples + 1);
+		fprintf(stderr, "  Width/Height/Depth       = %d/%d/%d\n", x, y,
+			gdk_drawable_get_depth (zoom_region->priv->w->window));
+		fprintf(stderr, "  Zoom Factor (x/y)        = %f/%f\n", zoom_region->xscale,
+			zoom_region->yscale);
+		fprintf(stderr, "  Pan Latency              = (avg. %f) (max. %f) seconds\n",
+			(mag_timing.idle_total / mag_timing.num_idle_samples), timing_idle_max);
+		fprintf(stderr, "  Total Frame Duration     = (avg. %f) (max. %f) (tot. %f) seconds\n",
+			frame_avg, timing_frame_max, mag_timing.frame_total);
+		fprintf(stderr, "  Frame Rate               = (avg. %f) (max. %f) frames/second\n",
+			1.0 / (mag_timing.frame_total / mag_timing.num_frame_samples), cps_max);
+		fprintf(stderr, "  Scroll Delta (x)         = (avg. %f) (tot. %d) lines\n",
+			x_scroll_incr, mag_timing.dx_total);
+		fprintf(stderr, "  Scroll Delta (y)         = (avg. %f) (tot. %d) lines\n",
+			y_scroll_incr, mag_timing.dy_total);
+		fprintf(stderr, "  Scroll Rate (x)          = (avg. %f) lines/second\n",
+			x_scroll_incr / frame_avg);
+		fprintf(stderr, "  Scroll Rate (y)          = (avg. %f) lines/second\n",
+			y_scroll_incr / frame_avg);
+
+		fprintf(stderr, "  Net Render Rate          = (avg. %f) (max. %f) Mpex/second\n\n",
+			(height * width *
+			((float)mag_timing.num_frame_samples / (float)mag_timing.frame_total)) / 1000000.0,
+			nrr_max / 1000000.0);
+	}
+}
+
 static void
 zoom_region_sync (ZoomRegion *zoom_region)
 {
@@ -2186,6 +2289,9 @@ zoom_region_get_property (BonoboPropertyBag *bag,
 	case ZOOM_REGION_MANAGED_PROP:
 		BONOBO_ARG_SET_BOOLEAN (arg, zoom_region->is_managed);
 		break;
+	case ZOOM_REGION_POLL_MOUSE_PROP:
+		BONOBO_ARG_SET_BOOLEAN (arg, zoom_region->poll_mouse);
+		break;
 	case ZOOM_REGION_INVERT_PROP:
 		BONOBO_ARG_SET_BOOLEAN (arg, zoom_region->invert);
 		break;
@@ -2263,6 +2369,25 @@ zoom_region_set_property (BonoboPropertyBag *bag,
 	case ZOOM_REGION_MANAGED_PROP:
 		zoom_region->is_managed = BONOBO_ARG_GET_BOOLEAN (arg);
 		break;
+	case ZOOM_REGION_POLL_MOUSE_PROP:
+		zoom_region->poll_mouse = BONOBO_ARG_GET_BOOLEAN (arg);
+		if (zoom_region->poll_mouse)
+		{
+		    g_message ("Adding polling timer");
+		    zoom_region->priv->update_pointer_id =
+			g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE,
+					    200,
+					    zoom_region_update_pointer_timeout,
+					    zoom_region,
+					    NULL);
+		}
+		else if (zoom_region->priv->update_pointer_id)
+		{
+		    g_message ("Removing polling timer");
+		    g_source_remove (zoom_region->priv->update_pointer_id);
+		    zoom_region->priv->update_pointer_id = 0;
+		}
+		break;
 	case ZOOM_REGION_INVERT_PROP:
 		zoom_region->invert = BONOBO_ARG_GET_BOOLEAN (arg);
 		zoom_region_update_current (zoom_region);
@@ -2311,12 +2436,12 @@ zoom_region_set_property (BonoboPropertyBag *bag,
 		break;
 	case ZOOM_REGION_XALIGN_PROP:
 		zoom_region->x_align_policy = BONOBO_ARG_GET_INT (arg);
-		zoom_region_align (zoom_region);
+		zoom_region_update_current (zoom_region);
 		break;
 	case ZOOM_REGION_YALIGN_PROP:
 		/* TODO: enums here */
 		zoom_region->y_align_policy = BONOBO_ARG_GET_INT (arg);
-		zoom_region_align (zoom_region);
+		zoom_region_update_current (zoom_region);
 		break;
 	case ZOOM_REGION_VIEWPORT_PROP:
 		bounds = BONOBO_ARG_GET_GENERAL (arg,
@@ -2447,6 +2572,47 @@ zoom_region_pan_test (gpointer data)
 	return FALSE;
 }
 
+static void
+impl_zoom_region_set_pointer_pos (PortableServer_Servant servant,
+				  const CORBA_long mouse_x,
+				  const CORBA_long mouse_y,
+				  CORBA_Environment *ev)
+{
+	ZoomRegion *zoom_region =
+		ZOOM_REGION (bonobo_object_from_servant (servant));
+	GdkRectangle paint_area, *clip = NULL;
+
+#ifdef ZOOM_REGION_DEBUG
+	g_assert (zoom_region->alive);
+#endif
+	DBG (fprintf (stderr, "Set Pointer: \t%ld,%ld\n", 
+		      (long) mouse_x, (long) mouse_y));
+
+	fprintf (stderr, "Set Pointer: \t%ld,%ld\n", 
+		      (long) mouse_x, (long) mouse_y);
+
+	zoom_region_set_cursor_pos (zoom_region, (int) mouse_x, (int) mouse_y);
+
+	if (GTK_IS_WIDGET (zoom_region->priv->w) && 
+	    GDK_IS_DRAWABLE (zoom_region->priv->w->window))
+	{
+	    gdk_drawable_get_size (
+		GDK_DRAWABLE (
+		    zoom_region->priv->w->window),
+		&paint_area.width, &paint_area.height);
+	    paint_area.x = 0;
+	    paint_area.y = 0;
+	    clip = &paint_area;
+	    paint_area = zoom_region_clip_to_source (
+		zoom_region, paint_area);
+	}
+	/* 
+	 * if we update the cursor now, it causes flicker if the client 
+	 * subsequently calls setROI, so we wait for a redraw.
+	 * Perhaps we should cue a redraw on idle instead?
+	 */
+}
+
 static void
 impl_zoom_region_set_roi (PortableServer_Servant servant,
 			  const GNOME_Magnifier_RectBounds *bounds,
@@ -2722,6 +2888,7 @@ zoom_region_class_init (ZoomRegionClass *klass)
 	epv->getMagFactor = impl_zoom_region_get_mag_factor;
 	epv->getProperties = impl_zoom_region_get_properties;
 	epv->setROI = impl_zoom_region_set_roi;
+	epv->setPointerPos = impl_zoom_region_set_pointer_pos;
 	epv->markDirty = impl_zoom_region_mark_dirty;
 	epv->getROI = impl_zoom_region_get_roi;
 	epv->moveResize = impl_zoom_region_move_resize;
@@ -2759,6 +2926,19 @@ zoom_region_properties_init (ZoomRegion *zoom_region)
 				 Bonobo_PROPERTY_READABLE |
 				 Bonobo_PROPERTY_WRITEABLE);
 
+	bonobo_arg_release (def);
+	def = bonobo_arg_new (BONOBO_ARG_BOOLEAN);
+	BONOBO_ARG_SET_BOOLEAN (def, TRUE);
+	
+	bonobo_property_bag_add (zoom_region->properties,
+				 "poll-mouse",
+				 ZOOM_REGION_POLL_MOUSE_PROP,
+				 BONOBO_ARG_BOOLEAN,
+				 NULL,
+				 "If false, zoom region does not poll for pointer location, but is (exclusively) given it by the client",
+				 Bonobo_PROPERTY_READABLE |
+				 Bonobo_PROPERTY_WRITEABLE);
+
 	bonobo_arg_release (def);
 	def = bonobo_arg_new (BONOBO_ARG_SHORT);
 	BONOBO_ARG_SET_SHORT (def, GNOME_Magnifier_ZoomRegion_SCROLL_FASTEST);
@@ -3016,6 +3196,7 @@ zoom_region_init (ZoomRegion *zoom_region)
 	zoom_region->x_align_policy = GNOME_Magnifier_ZoomRegion_ALIGN_CENTER;
 	zoom_region->y_align_policy = GNOME_Magnifier_ZoomRegion_ALIGN_CENTER;
 	zoom_region->coalesce_func = _coalesce_update_rects;
+	zoom_region->poll_mouse = TRUE; 
 	zoom_region->priv = g_malloc (sizeof (ZoomRegionPrivate));
 	zoom_region_private_init (zoom_region->priv);
 	bonobo_object_add_interface (BONOBO_OBJECT (zoom_region),
